/*
 * CreateFleet.cpp
 *
 * (C) 2001 MagnumMan-iCoP (James E. King, III)
 */

#include "StdAfx.h"

#include "ShipList.h"

// #pragma warning( disable:4715 )

//
// Logic...
//

ShipList ShipList::CreateFleet(int32 totalShips, int32 totalBPV)
{
	ShipList	sl;

	sl = *this;						// copy the fighters_and_pfs along with the ship list
	sl.erase(sl.begin(), sl.end());	// empty the ship list, leave fighters_and_pfs

	if (!size())
		return sl;					// There are no ships to select from, return empty list
	
	//
	// Ensure there is enough bpv for the number of ships selectable from the
	// current list.
	//
	// For example, if the list is only BB (say 400+ bpv) and you call
	// CreateFleet(1, 200) then it should return a list with size() == 0.
	// CreateFleet(2, 900) should be ok.
	//

	if (totalBPV < totalShips * MinimumBPV())
		return sl;
	
	for (int k = 1; k <= totalShips; k++)
	{
		Ship s;

		if (!SelectRandomShip(s))
		{
			sl.erase(sl.begin(), sl.end());	// empty the ship list, leave fighters_and_pfs
			return sl;						// return the empty list
		}

		//
		// Now we have to check and make sure there is sufficient BPV left over
		// If not, try again.
		//

		if (totalBPV - s.bpv < (totalShips - k) * MinimumBPV() ||
			totalBPV - s.bpv < 0)
		{
			k--;
			continue;						// Not enough bpv left over for the other ships
		}
		
		totalBPV -= s.bpv;

		sl.push_back(s);
	}

	return sl;
}
