/*
 * Decode.cpp
 *
 * (C) 2001 MagnumMan-iCoP (James E. King, III)
 */

#include "StdAfx.h"

#include "ShipList.h"
#include "EnhancedShiplist.h"
#include "Tokenizer.h"

eRace ShipList::decodeRace(const char *raceStr)
{
	if (!stricmp(raceStr, "Federation")) return kFederation;
	if (!stricmp(raceStr, "Klingon")) return kKlingon;
	if (!stricmp(raceStr, "Romulan")) return kRomulan;
	if (!stricmp(raceStr, "Lyran")) return kLyran;
	if (!stricmp(raceStr, "Hydran")) return kHydran;
	if (!stricmp(raceStr, "Gorn")) return kGorn;
	if (!stricmp(raceStr, "ISC")) return kISC;
	if (!stricmp(raceStr, "Mirak")) return kMirak;
	if (!stricmp(raceStr, "Orion")) return kOrion;
	if (!stricmp(raceStr, "Monster")) return kMonster;
	if (!stricmp(raceStr, "Neutral")) return kNeutralRace;

#ifdef OP
	// TODO 20010608 JEK done locally
	if (!stricmp(raceStr, "OrionOrion")) return kOrionOrion;
	if (!stricmp(raceStr, "OrionKorgath")) return kOrionKorgath;
	if (!stricmp(raceStr, "OrionPrime")) return kOrionPrime;
	if (!stricmp(raceStr, "OrionTigerHeart")) return kOrionTigerHeart;
	if (!stricmp(raceStr, "OrionBeastRaiders")) return kOrionBeastRaiders;
	if (!stricmp(raceStr, "OrionSyndicate")) return kOrionSyndicate;
	if (!stricmp(raceStr, "OrionWyldeFire")) return kOrionWyldeFire;
	if (!stricmp(raceStr, "OrionCamboro")) return kOrionCamboro;
#endif

	if (raceStr[0] == 0) return kNoRaceType;

#ifdef _DEBUG
	DebugBreak();					// Should not occur
#endif

	return kUndefinedRace;
}

eHullType ShipList::decodeHullType(const char *hullStr)
{
	if (!stricmp(hullStr, "FF")) return kFF;
	if (!stricmp(hullStr, "DD")) return kDD;
	if (!stricmp(hullStr, "CL")) return kCL;
	if (!stricmp(hullStr, "CA")) return kCA;
	if (!stricmp(hullStr, "DN")) return kDN;
	if (!stricmp(hullStr, "F")) return kF;
	if (!stricmp(hullStr, "SB")) return kSB;
	if (!stricmp(hullStr, "BS")) return kBS;
	if (!stricmp(hullStr, "BT")) return kBT;
	if (!stricmp(hullStr, "KLP")) return kHullListeningPost;
	if (!stricmp(hullStr, "KDP")) return kHullDefensePlatform;
	if (!stricmp(hullStr, "KSD")) return kHullStarDock;
	if (!stricmp(hullStr, "MAM")) return kHullAstroMiner;
	if (!stricmp(hullStr, "MSG")) return kHullSunGlider;
	if (!stricmp(hullStr, "MDM")) return kHullDoomsdayMachine;
	if (!stricmp(hullStr, "MLC")) return kHullLivingCage;
	if (!stricmp(hullStr, "MMT")) return kHullM_Eater;
	if (!stricmp(hullStr, "MSS")) return kHullSpaceShell;
	if (!stricmp(hullStr, "Planet")) return kHullPlanet;
	if (!stricmp(hullStr, "Asteroid")) return kHullAsteroid;
	if (!stricmp(hullStr, "Star")) return kHullStar;
#ifdef OP
	if (!stricmp(hullStr, "BOX")) return kBox;
#endif

	if (!stricmp(hullStr, "XMN")) return kMineHull;
	if (!stricmp(hullStr, "FTR")) return kFighter;
	if (!stricmp(hullStr, "SHT")) return kShuttle;

	// TODO 20010608 JEK done locally
#ifdef OP
	if (!stricmp(hullStr, "AB")) return kHullAsteroidBase;
	if (!stricmp(hullStr, "MIT")) return kHullIntruder;
#endif

	if (!stricmp(hullStr, "Unknown")) return kUnknownHull;

#ifdef _DEBUG
	DebugBreak();					// Should not occur
#endif

	return kUnknownHull;
}

eClassTypes	ShipList::decodeShipClass(const char *classStr)
{
	if (!stricmp(classStr, "SHUTTLE")) return kClassShuttle;
	if (!stricmp(classStr, "PF")) return kClassPseudoFighter;
	if (!stricmp(classStr, "FREIGHTER")) return kClassFreighter;
	if (!stricmp(classStr, "FRIGATE")) return kClassFrigate;
	if (!stricmp(classStr, "DESTROYER")) return kClassDestroyer;
	if (!stricmp(classStr, "WAR_DESTROYER")) return kClassWarDestroyer;
	if (!stricmp(classStr, "LIGHT_CRUISER")) return kClassLightCruiser;
	if (!stricmp(classStr, "HEAVY_CRUISER")) return kClassHeavyCruiser;
	if (!stricmp(classStr, "NEW_HEAVY_CRUISER")) return kClassNewHeavyCruiser;
	if (!stricmp(classStr, "HEAVY_BATTLECRUISER")) return kClassHeavyBattlecruiser;
	if (!stricmp(classStr, "CARRIER")) return kClassCarrier;
	if (!stricmp(classStr, "DREADNOUGHT")) return kClassDreadnought;
	if (!stricmp(classStr, "BATTLESHIP")) return kClassBattleship;
	if (!stricmp(classStr, "LISTENING_POST")) return kClassListeningPost;
	if (!stricmp(classStr, "BASE_STATION")) return kClassBaseStation;
	if (!stricmp(classStr, "BATTLE_STATION")) return kClassBattleStation;
	if (!stricmp(classStr, "STARBASE")) return kClassStarBase;
	if (!stricmp(classStr, "MONSTER")) return kClassMonster;
	if (!stricmp(classStr, "PLANET")) return kClassPlanets;
	if (!stricmp(classStr, "SPECIAL")) return kClassSpecial;
	if (!stricmp(classStr, "NONE")) return kNoClassType;

#ifdef _DEBUG
	DebugBreak();					// Should not occur
#endif

	return kNoClassType;
}


// Enhanced ClassTypes. --FS
eEnhancedClassTypes	ShipList::decodeEnhancedShipClass(const char *classStr)
{
	// Attrition Units
	if (!stricmp(classStr, "SHUTTLE")) return kEClassShuttle;
	if (!stricmp(classStr, "PF")) return kEClassPseudoFighter;

	// Civilian and pirate free traders
	if (!stricmp(classStr, "CIVILIAN_PASSENGER_FREIGHTER")) return kEClassCivilianPassengerFreighter;
	if (!stricmp(classStr, "CIVILIAN_FREIGHTER")) return kEClassCivilianFreighter;
	if (!stricmp(classStr, "PIRATE_FREIGHTER")) return kEClassPirateFreighter;

	// Freighters
	if (!stricmp(classStr, "SMALL_FREIGHTER")) return kEClassSmallFreighter;
	if (!stricmp(classStr, "LARGE_FREIGHTER")) return kEClassLargeFreighter;
	if (!stricmp(classStr, "HEAVY_FREIGHTER")) return kEClassHeavyFreighter;

	// Auxiliaries
	if (!stricmp(classStr, "SMALL_AUXILIARY")) return kEClassSmallAuxiliary;
	if (!stricmp(classStr, "LARGE_AUXILIARY")) return kEClassLargeAuxiliary;
	if (!stricmp(classStr, "HEAVY_AUXILIARY")) return kEClassHeavyAuxiliary;

	// Tugs
	if (!stricmp(classStr, "HARBOR_TUG")) return kEClassHarborTug;
	if (!stricmp(classStr, "LIGHT_TUG")) return kEClassLightTug;
	if (!stricmp(classStr, "TUG")) return kEClassTug;
	if (!stricmp(classStr, "NEW_TUG")) return kEClassNewTug;

	// Ships
	if (!stricmp(classStr, "FRIGATE")) return kEClassFrigate;
	if (!stricmp(classStr, "DESTROYER")) return kEClassDestroyer;
	if (!stricmp(classStr, "WAR_DESTROYER")) return kEClassWarDestroyer;
	if (!stricmp(classStr, "LIGHT_CRUISER")) return kEClassLightCruiser;
	if (!stricmp(classStr, "WAR_CRUISER")) return kEClassWarCruiser;
	if (!stricmp(classStr, "HEAVY_WAR_DESTROYER")) return kEClassHeavyWarDestroyer;
	if (!stricmp(classStr, "MEDIUM_CRUISER")) return kEClassMediumCruiser;
	if (!stricmp(classStr, "HEAVY_CRUISER")) return kEClassHeavyCruiser;
	if (!stricmp(classStr, "NEW_HEAVY_CRUISER")) return kEClassNewHeavyCruiser;
	if (!stricmp(classStr, "HEAVY_BATTLECRUISER")) return kEClassHeavyBattlecruiser;
	if (!stricmp(classStr, "MONITOR")) return kEClassMonitor;
	if (!stricmp(classStr, "DREADNOUGHT")) return kEClassDreadnought;
	if (!stricmp(classStr, "HEAVY_DREADNOUGHT")) return kEClassHeavyDreadnought;
	if (!stricmp(classStr, "BATTLESHIP")) return kEClassBattleship;

	// Bases
	if (!stricmp(classStr, "LISTENING_POST")) return kEClassListeningPost;
	if (!stricmp(classStr, "PLATFORM")) return kEClassPlatform;
	if (!stricmp(classStr, "FLEET_REPAIR_DOCK")) return kEClassFleetRepairDock;
	if (!stricmp(classStr, "MOBILE_BASE")) return kEClassMobileBase;
	if (!stricmp(classStr, "BASE_STATION")) return kEClassBaseStation;
	if (!stricmp(classStr, "BATTLE_STATION")) return kEClassBattleStation;
	if (!stricmp(classStr, "SECTOR_BASE")) return kEClassSectorBase;
	if (!stricmp(classStr, "STARBASE")) return kEClassStarBase;
	
	// Monsters
	if (!stricmp(classStr, "DEAD_MONSTER")) return kEClassDeadMonster;
	if (!stricmp(classStr, "SMALL_MONSTER")) return kEClassSmallMonster;
	if (!stricmp(classStr, "MEDIUM_MONSTER")) return kEClassMediumMonster;
	if (!stricmp(classStr, "LARGE_MONSTER")) return kEClassLargeMonster;

	// misc
	if (!stricmp(classStr, "BOX")) return kEClassBox;
	if (!stricmp(classStr, "SMALL_ASTEROID")) return kEClassSmallAsteroid;
	if (!stricmp(classStr, "LARGE_ASTEROID")) return kEClassLargeAsteroid;
	if (!stricmp(classStr, "PLANET")) return kEClassPlanet;
	if (!stricmp(classStr, "STAR")) return kEClassStar;
	if (!stricmp(classStr, "UNKNOWN")) return kEClassUnknown;
	if (!stricmp(classStr, "NONE")) return kNoEClassType;

#ifdef _DEBUG
	DebugBreak();					// Should not occur
#endif

	return kNoEClassType;
}


eEnhancedRoles ShipList::decodeEnhancedRole(const char *roleStr)
{
	if (roleStr[0] == 0) return kNoEnhancedRole;

	if (!stricmp(roleStr, "LINE_SHIP")) return kLineShip;
	if (!stricmp(roleStr, "WARSHIP")) return kWarShip;
	if (!stricmp(roleStr, "LEADER")) return kLeader;
	if (!stricmp(roleStr, "DRONE")) return kDroneBombardmentShip;
	if (!stricmp(roleStr, "ESCORT")) return kEscort;
	if (!stricmp(roleStr, "CARRIER")) return kCarrier;
	if (!stricmp(roleStr, "PF_TENDER")) return kPFTender;
	if (!stricmp(roleStr, "FAST_CRUISER")) return kFastCruiser;
	if (!stricmp(roleStr, "MAULER")) return kMauler;
	if (!stricmp(roleStr, "POLICE")) return kPolice;
	if (!stricmp(roleStr, "REPAIR")) return kRepair;
	if (!stricmp(roleStr, "COMMANDO")) return kCommando;
	if (!stricmp(roleStr, "HOSPITAL")) return kHospital;
	if (!stricmp(roleStr, "SCOUT")) return kScout;
	if (!stricmp(roleStr, "SURVEY")) return kSurvey;
	if (!stricmp(roleStr, "BASE")) return kBase;
	if (!stricmp(roleStr, "LISTENING_POST")) return kListeningPost;
	if (!stricmp(roleStr, "PLANET")) return kPlanet;
	if (!stricmp(roleStr, "HOMEWORLD")) return kHomeWorld;
	if (!stricmp(roleStr, "ARMED_WORLD")) return kArmedWorld;
	if (!stricmp(roleStr, "SHUTTLE")) return kRoleShuttle;
	if (!stricmp(roleStr, "ASTEROID")) return kAsteroid;
	if (!stricmp(roleStr, "MONSTER")) return kRoleMonster;
	if (!stricmp(roleStr, "TOURNAMENT")) return kTournament;
	if (!stricmp(roleStr, "NON_COMBATANT")) return kNonCombatant;
	if (!stricmp(roleStr, "CARGO")) return kCargo;
	if (!stricmp(roleStr, "QSHIP")) return kQship;
	if (!stricmp(roleStr, "LOCAL_DEFENSE")) return kLocalDefense;
	if (!stricmp(roleStr, "SPECIAL")) return kSpecial;

	if (!stricmp(roleStr, "WYN")) return kRoleWYN;
	if (!stricmp(roleStr, "LDR")) return kRoleLDR;
	if (!stricmp(roleStr, "CUSTOMRACE1")) return kRoleCustomRace1;
	if (!stricmp(roleStr, "CUSTOMRACE2")) return kRoleCustomRace2;
	if (!stricmp(roleStr, "CUSTOMRACE3")) return kRoleCustomRace3;
	if (!stricmp(roleStr, "CUSTOMRACE4")) return kRoleCustomRace4;
	if (!stricmp(roleStr, "CUSTOMRACE5")) return kRoleCustomRace5;
	if (!stricmp(roleStr, "CUSTOMRACE6")) return kRoleCustomRace6;
	if (!stricmp(roleStr, "CUSTOMRACE7")) return kRoleCustomRace7;
	if (!stricmp(roleStr, "CUSTOMRACE8")) return kRoleCustomRace8;
	if (!stricmp(roleStr, "CUSTOMRACE9")) return kRoleCustomRace9;
	if (!stricmp(roleStr, "CUSTOMRACE10")) return kRoleCustomRace10;
	if (!stricmp(roleStr, "CUSTOMRACE11")) return kRoleCustomRace11;
	if (!stricmp(roleStr, "CUSTOMRACE12")) return kRoleCustomRace12;
	if (!stricmp(roleStr, "CUSTOMRACE13")) return kRoleCustomRace13;
	if (!stricmp(roleStr, "CUSTOMRACE14")) return kRoleCustomRace14;
	if (!stricmp(roleStr, "CUSTOMRACE15")) return kRoleCustomRace15;
	if (!stricmp(roleStr, "CUSTOMRACE16")) return kRoleCustomRace16;
	if (!stricmp(roleStr, "CUSTOMRACE17")) return kRoleCustomRace17;
	if (!stricmp(roleStr, "CUSTOMRACE18")) return kRoleCustomRace18;
	if (!stricmp(roleStr, "CUSTOMRACE19")) return kRoleCustomRace19;

	return kNoEnhancedRole;
}

eEnhancedXTech ShipList::decodeEnhancedXTech(const char *xtechStr)
{
	if (xtechStr[0] == 0) return kNoXtech;

	if (!stricmp(xtechStr, "X1")) return kX1;
	if (!stricmp(xtechStr, "X2")) return kX2;

	return kNoXtech;
}

eEnhancedProductionAvailability ShipList::decodeProductionAvailability(const char *availStr)
{
	if (availStr[0] == 0) return kNoProductionAvailability;

	if (!stricmp(availStr, "NONE")) return kConjectural;
	if (!stricmp(availStr, "UNIQUE")) return kUnique;
	if (!stricmp(availStr, "RARE")) return kRare;
	if (!stricmp(availStr, "COMMON")) return kCommon;
	if (!stricmp(availStr, "MANY")) return kMany;

	return kNoProductionAvailability;
}


eSpecialRole ShipList::decodeSpecialRole(const char *roleStr)
{
	if (roleStr[0] == 0) return kNoSpecialRole;
	if (!stricmp(roleStr, "C")) return kSpecialRoleC;
	if (!stricmp(roleStr, "D")) return kSpecialRoleD;
	if (!stricmp(roleStr, "K")) return kSpecialRoleK;
	if (!stricmp(roleStr, "KM")) return kSpecialRoleKM;
	if (!stricmp(roleStr, "M")) return kSpecialRoleM;
	if (!stricmp(roleStr, "S")) return kSpecialRoleS;
	if (!stricmp(roleStr, "R")) return kSpecialRoleR;
	if (!stricmp(roleStr, "NT")) return kSpecialRoleNT;

	if (!stricmp(roleStr, "E")) return kSpecialRoleE;		// kSpecialRoleE ???????????
	if (!stricmp(roleStr, "I")) return kSpecialRoleI;		// kSpecialRoleI ???????????
	if (!stricmp(roleStr, "L")) return kSpecialRoleL;		// kSpecialRoleL ???????????
	if (!stricmp(roleStr, "P")) return kSpecialRoleP;		// kSpecialRoleP ???????????
	if (!stricmp(roleStr, "Q")) return kSpecialRoleQ;		// kSpecialRoleQ ???????????
	if (!stricmp(roleStr, "T")) return kSpecialRoleT;		// kSpecialRoleT ???????????
	if (!stricmp(roleStr, "V")) return kSpecialRoleV;		// kSpecialRoleV ???????????
	if (!stricmp(roleStr, " ")) return kSpecialRolePuck;	// kSpecialRolePuck ???????????

	//TODO
	if (!stricmp(roleStr, "A")) return kSpecialRoleA;		// kSpecialRoleA ???????????

#ifdef _DEBUG
	DebugBreak();					// Should not occur
#endif

	return kNoSpecialRole;
}

eTurnMode ShipList::decodeTurnMode(const char *turnMode)
{
	if (!*turnMode) return kNoTurnMode;
	if (!stricmp(turnMode, "AA")) return kTurnModeAA;
	if (*turnMode == 'A') return kTurnModeA;
	if (*turnMode == 'B') return kTurnModeB;
	if (*turnMode == 'C') return kTurnModeC;
	if (*turnMode == 'D') return kTurnModeD;
	if (*turnMode == 'E') return kTurnModeE;
	if (*turnMode == 'F') return kTurnModeF;
	if (*turnMode == 'G') return kTurnModeG;
	if (*turnMode == 'H') return kTurnModeH;


#ifdef _DEBUG
	DebugBreak();					// Should not occur
#endif

	return kNoTurnMode;
}

eShipSystemType ShipList::decodeWeaponType(const char *weaponStr, bool isFighter)
{
	if (!*weaponStr) return kNoWeaponType;

	if (!stricmp(weaponStr, "Phot")) return kShipSystemPhoton;
	if (!stricmp(weaponStr, "PhoF")) return kShipSystemPhotonFighter;

	if (!stricmp(weaponStr, "Dis1")) return kShipSystemDisruptor1;
	if (!stricmp(weaponStr, "Dis2")) return kShipSystemDisruptor2;
	if (!stricmp(weaponStr, "Dis3")) return kShipSystemDisruptor3;
	if (!stricmp(weaponStr, "Dis4")) return kShipSystemDisruptor4;
	if (!stricmp(weaponStr, "DisF")) return kShipSystemDisruptorFighter;

	if (!stricmp(weaponStr, "Fus")) return kShipSystemFusion;
	if (!stricmp(weaponStr, "FusF")) return kShipSystemFusionFighter;
	
	if (!stricmp(weaponStr, "HB")) return kShipSystemHellbore;
	if (!stricmp(weaponStr, "HellF")) return kShipSystemHellboreFighter;

	if (!stricmp(weaponStr, "PlaR")) return kShipSystemPlasmaR;
	if (!stricmp(weaponStr, "PlaS")) return kShipSystemPlasmaS;
	if (!stricmp(weaponStr, "PlaG")) return kShipSystemPlasmaG;
	if (!stricmp(weaponStr, "PlaF")) return kShipSystemPlasmaF;
	if (!stricmp(weaponStr, "PlaI")) return kShipSystemPlasmaI;
	if (!stricmp(weaponStr, "PlaD")) return kShipSystemPlasmaD;

	if (!stricmp(weaponStr, "ESG")) return kShipSystemESG;
	if (!stricmp(weaponStr, "PPD")) return kShipSystemPPD;

//	kShipSystemShuttleWeapon,
//	kShipSystemFighterSystem,

	if (!stricmp(weaponStr, "DroA")) return kShipSystemDroneRackA;
	if (!stricmp(weaponStr, "DroB")) return kShipSystemDroneRackB;
	if (!stricmp(weaponStr, "DroC")) return kShipSystemDroneRackC;
	if (!stricmp(weaponStr, "DroD")) return kShipSystemDroneRackD;
	if (!stricmp(weaponStr, "DroE")) return kShipSystemDroneRackE;
	if (!stricmp(weaponStr, "DroF")) return kShipSystemDroneRackF;
	if (!stricmp(weaponStr, "DroG")) return kShipSystemDroneRackG;
	if (!stricmp(weaponStr, "DroH")) return kShipSystemDroneRackH;
	if (!stricmp(weaponStr, "DroI")) return kShipSystemDroneFighter1;
	if (!stricmp(weaponStr, "DroVI")) return kShipSystemDroneFighter4;

//	kShipSystemOptional,
//	kShipSystemOptionalW,

	if (!stricmp(weaponStr, "Ph1")) return kShipSystemPhaser1;
	if (!stricmp(weaponStr, "Ph2")) return kShipSystemPhaser2;
	if (!stricmp(weaponStr, "Ph3")) return kShipSystemPhaser3;
	if (!stricmp(weaponStr, "PhG")) return kShipSystemPhaserG;
	if (!stricmp(weaponStr, "Ph4")) return kShipSystemPhaser4;

	if (!stricmp(weaponStr, "ADD6")) return kShipSystemADD6;
	if (!stricmp(weaponStr, "ADD12")) return kShipSystemADD12;
	if (!stricmp(weaponStr, "ADD30")) return kShipSystemADD30;

//	TODO 2001/06/08 added locally
#ifdef OP
	if (!stricmp(weaponStr, "PhA")) return kShipSystemPhaserA;
	if (!stricmp(weaponStr, "PhB")) return kShipSystemPhaserB;
	if (!stricmp(weaponStr, "PhX")) return kShipSystemPhaserX;
	if (!stricmp(weaponStr, "PhG2")) return kShipSystemPhaserG2;
	if (!stricmp(weaponStr, "ESGL")) return kShipSystemESGLance;
	if (!stricmp(weaponStr, "PhoH")) return kShipSystemHeavyPhoton;
	if (!stricmp(weaponStr, "DisH")) return kShipSystemHeavyDisruptor;
	if (!stricmp(weaponStr, "DroM")) return kShipSystemDroneRackMIRV;
	if (!stricmp(weaponStr, "PlaX")) return kShipSystemPlasmaX;
	if (!stricmp(weaponStr, "PlaE")) return kShipSystemPlasmaE;
	if (!stricmp(weaponStr, "MLR")) return kShipSystemMauler;
#endif

#ifdef _DEBUG
	DebugBreak();					// Should not occur
#endif

	return kNoWeaponType;
}

eWeaponArc ShipList::decodeWeaponArc(const char *arcStr)
{
	if (!*arcStr) return kNoWeaponArc;

	if (!stricmp(arcStr, "1")) return k1DroneReloadArc360;
	if (!stricmp(arcStr, "2")) return k2DroneReloadArc360;
	if (!stricmp(arcStr, "3")) return k3DroneReloadArc360;

// Mirak ships have these weapon arcs which are not in the API?

	if (!stricmp(arcStr, "FXL")) return kWeaponArcFXL; // Added to API
	if (!stricmp(arcStr, "FXR")) return kWeaponArcFXR;
	if (!stricmp(arcStr, "RXL")) return kWeaponArcRXL;
	if (!stricmp(arcStr, "RXR")) return kWeaponArcRXR;
	
	if (!stricmp(arcStr, "FA")) return kWeaponArcFA;
	if (!stricmp(arcStr, "RS")) return kWeaponArcRS;
	if (!stricmp(arcStr, "FRR")) return kWeaponArcFRR;
	if (!stricmp(arcStr, "FLL")) return kWeaponArcFLL;
	if (!stricmp(arcStr, "FH")) return kWeaponArcFH;
	if (!stricmp(arcStr, "RH")) return kWeaponArcRH;
	if (!stricmp(arcStr, "LS")) return kWeaponArcLS;
	if (!stricmp(arcStr, "FX")) return kWeaponArcFX;
	if (!stricmp(arcStr, "RX")) return kWeaponArcRX;
	if (!stricmp(arcStr, "RRR")) return kWeaponArcRRR;
	if (!stricmp(arcStr, "LLR")) return kWeaponArcLLR;
	if (!stricmp(arcStr, "FAR")) return kWeaponArcFAR;
	if (!stricmp(arcStr, "FAL")) return kWeaponArcFAL;
	if (!stricmp(arcStr, "ALL")) return kWeaponArcALL;
	if (!stricmp(arcStr, "RW")) return kWeaponArcRW;
	if (!stricmp(arcStr, "LW")) return kWeaponArcLW;
	if (!stricmp(arcStr, "FHR")) return kWeaponArcFHR;
	if (!stricmp(arcStr, "FHL")) return kWeaponArcFHL;
	if (!stricmp(arcStr, "RA")) return kWeaponArcRA;
	if (!stricmp(arcStr, "RWX")) return kWeaponArcRWX;
	if (!stricmp(arcStr, "LWX")) return kWeaponArcLWX;
	if (!stricmp(arcStr, "RP")) return kWeaponArcRP;
	if (!stricmp(arcStr, "LP")) return kWeaponArcLP;
	if (!stricmp(arcStr, "RAR")) return kWeaponArcRAR;
	if (!stricmp(arcStr, "RAL")) return kWeaponArcRAL;
	if (!stricmp(arcStr, "FLLX")) return kWeaponArcFLLX;
	if (!stricmp(arcStr, "FRRX")) return kWeaponArcFRRX;
		
	// Added to local API 2001/06/08

#ifdef OP
	if (!stricmp(arcStr, "MLR")) return kWeaponArcMLR;
#endif
	if (!stricmp(arcStr, "FARA")) return kWeaponArcFARA;
	if (!stricmp(arcStr, "KFX")) return kWeaponArcKFX;

#ifdef _DEBUG
	DebugBreak();					// Should not occur
#endif

	return kNoWeaponArc;
}

bool	ShipList::decodeShip(const char *data, Ship &aShip, bool pfRun)
{
	Tokenizer	tok(data, "\t");

	aShip.race = decodeRace(tok.getNextTokenC());
	if (aShip.race == kNoRaceType)
		return false;

	aShip.hullType = decodeHullType(tok.getNextTokenC());
	aShip.shipCode = tok.getNextTokenC();
	aShip.shipClass = decodeShipClass(tok.getNextTokenC());

	if (pfRun && aShip.shipClass != kClassPseudoFighter)
		return false;
	if (!pfRun && aShip.shipClass == kClassPseudoFighter)
		return false;

	aShip.bpv = tok.getNextTokenI();
	aShip.specialRole = decodeSpecialRole(tok.getNextTokenC());
	aShip.yearFirstAvail = tok.getNextTokenI();
	aShip.yearLastAvail = tok.getNextTokenI();
	aShip.sizeClass = tok.getNextTokenI();
	aShip.turnMode = decodeTurnMode(tok.getNextTokenC());
	aShip.moveCost = tok.getNextTokenF();
	aShip.hetPlusTwo = tok.getNextTokenI();
	aShip.hetBreakdown = tok.getNextTokenI();
	aShip.stealthECM = tok.getNextTokenI();
	aShip.regularCrew = tok.getNextTokenI();
	aShip.marinesBase = tok.getNextTokenI();
	aShip.marinesMax = tok.getNextTokenI();
	aShip.deckCrew = tok.getNextTokenI();
	aShip.totalCrew = tok.getNextTokenI();
	aShip.minimumCrew = tok.getNextTokenI();

	aShip.shields[kForeShieldIndex] = tok.getNextTokenI();
	aShip.shields[kLeftForeShieldIndex] = aShip.shields[kRightForeShieldIndex] = tok.getNextTokenI();
	aShip.shields[kLeftAftShieldIndex] = aShip.shields[kRightAftShieldIndex] = tok.getNextTokenI();
	aShip.shields[kAftShieldIndex] = tok.getNextTokenI();
	aShip.totalShielding = tok.getNextTokenI();

	aShip.cloakCost = tok.getNextTokenI();

	for (int hpi = (int)kNewFirstWeaponHardPoint; hpi <= (int)kNewLastWeaponHardPoint; hpi++)
	{
		WeaponMount		wm;

		wm.count = tok.getNextTokenI();
		wm.weaponTypeStr = tok.getNextTokenC();
		wm.weaponType = decodeWeaponType(wm.weaponTypeStr.c_str());
		wm.weaponArcStr = tok.getNextTokenC();
		wm.weaponArc = decodeWeaponArc(wm.weaponArcStr.c_str());
		wm.shots = 0;			// irrelevant to "ships" - only fighters

		aShip.weapons[(eSystemHardPoint)hpi] = wm;
	}

	aShip.probes = tok.getNextTokenI();
	aShip.tBombsBase = tok.getNextTokenI();
	aShip.tBombsMax = tok.getNextTokenI();
	aShip.nukeBombsBase = tok.getNextTokenI();
	aShip.nukeBombsMax = tok.getNextTokenI();
	aShip.maxDroneControl = tok.getNextTokenI();
	aShip.add_6 = tok.getNextTokenI();
	aShip.add_12 = tok.getNextTokenI();
	aShip.shuttlesSize = tok.getNextTokenI();
	aShip.launchRate = tok.getNextTokenI();
	aShip.adminBase = tok.getNextTokenI();
	aShip.adminMax = tok.getNextTokenI();
	
	// 4 fighter slots
	for (int k = 0; k < 4; k++)
	{
		int32	count;
		Ship	ship;

		count = tok.getNextTokenI();
		if (count)
		{
			if (!FindFighter(tok.getNextTokenC(), ship)) 
				return false;
			aShip.fighters[k] = Fighters(ship, count);
		}
		else
			tok.skipNextToken();	// skip
	}

	aShip.armor = tok.getNextTokenI();
	aShip.forwardHull = tok.getNextTokenI();
	aShip.centerHull = tok.getNextTokenI();
	aShip.aftHull = tok.getNextTokenI();
	aShip.cargo = tok.getNextTokenI();
	aShip.barracks = tok.getNextTokenI();
	aShip.repair = tok.getNextTokenI();
	aShip.rightWarp = aShip.leftWarp = tok.getNextTokenI();
	aShip.centerWarp = tok.getNextTokenI();
	aShip.impulse = tok.getNextTokenI();
	aShip.apr = tok.getNextTokenI();
	aShip.battery = tok.getNextTokenI();
	aShip.bridge = tok.getNextTokenI();
	aShip.security = tok.getNextTokenI();
	aShip.lab = tok.getNextTokenI();
	aShip.transporters = tok.getNextTokenI();
	aShip.tractors = tok.getNextTokenI();
	aShip.mechTractors = tok.getNextTokenI();
	aShip.specialSensors = tok.getNextTokenI();
	aShip.sensors = tok.getNextTokenI();
	aShip.scanners = tok.getNextTokenI();
	aShip.explosionStrength = tok.getNextTokenI();
	aShip.acceleration = tok.getNextTokenI();
	aShip.damageControl = tok.getNextTokenI();
	aShip.extraDamage = tok.getNextTokenI();
	aShip.shipCost = tok.getNextTokenI();

	aShip.baseRefitClass = tok.getNextTokenC();
	aShip.geometry = tok.getNextTokenC();
	aShip.geoUI = tok.getNextTokenC();
	aShip.fullName = tok.getNextTokenC();
	aShip.refits = tok.getNextTokenC();
	aShip.balance = tok.getNextTokenI();

	// then just some excel calculations and unused columns.
	/*
	151 -- refits
	152 -- Balance
	153 -- End -- 1
	154 -- 
	155 -- W1
	156 -- W2
	157 -- W3
	158 -- W4
	159 -- W5
	160 -- W6
	161 -- W7
	162 -- W8
	163 -- W9
	164 -- W10
	165 -- W11
	166 -- W12
	167 -- W13
	168 -- W14
	169 -- W15
	170 -- W16
	171 -- W17
	172 -- W18
	173 -- W19
	174 -- W20
	175 -- W21
	176 -- W22
	177 -- W23
	178 -- W24
	179 -- W25
	180 -- Weapon BPV
	181 -- FB1
	182 -- FB2
	183 -- FB3
	184 -- FB4
	185 -- Fighter Subtotal
	186 -- System BPV
	187 -- BPV Subtotal
	188 -- YFA
	189 -- YLA -- 37
	*/

	// These must be read to be able to reach the enhanced shiplist columns. -- FS
	for (int i = 1; i <= 37; i++) {
		tok.skipNextToken(); // skip
	}


	// enhanced shiplist columns. -- FS
	/*
	190 -- Enhanced Class Type
	191 -- Role
	192 -- Secondary Role
	193 -- Xtech
	*/
	aShip.enhancedShipClass = decodeEnhancedShipClass(tok.getNextTokenC());
	aShip.enhancedRole = decodeEnhancedRole(tok.getNextTokenC());
	aShip.enhancedSecondaryRole = decodeEnhancedRole(tok.getNextTokenC());
	aShip.enhancedXtech = decodeEnhancedXTech(tok.getNextTokenC());
	aShip.enhancedProductionAvailability = decodeProductionAvailability(tok.getNextTokenC());

	return true;
}

bool	ShipList::decodeFighter(const char *data, Ship &aShip)
{
	Tokenizer	tok(data, "\t");
	int			hpi;

	aShip.race = decodeRace(tok.getNextTokenC());
	if (aShip.race == kNoRaceType)
		return false;

	aShip.shipCode = tok.getNextTokenC();
	aShip.speed = tok.getNextTokenI();
	for (hpi = (int)kHeavyWeapon1HardPoint; hpi <= (int)kHeavyWeapon5HardPoint; hpi++)
	{
		WeaponMount		wm;

		wm.count = tok.getNextTokenI();
		wm.weaponTypeStr = tok.getNextTokenC();
		wm.weaponType = decodeWeaponType(wm.weaponTypeStr.c_str());
		wm.weaponArcStr = tok.getNextTokenC();
		wm.weaponArc = decodeWeaponArc(wm.weaponArcStr.c_str());
		wm.shots = tok.getNextTokenI();

		aShip.weapons[(eSystemHardPoint)hpi] = wm;
	}
	
	for (hpi = (int)kHeavyWeapon6HardPoint; hpi <= (int)kNewLastWeaponHardPoint; hpi++)
	{
		WeaponMount	wm;
		memset(&wm, 0, sizeof(WeaponMount));
		aShip.weapons[(eSystemHardPoint)hpi] = wm;
	}

	aShip.damage = tok.getNextTokenI();
	aShip.add_6 = tok.getNextTokenI();
	aShip.groundAttackBonus = tok.getNextTokenI();
	aShip.stealthECM = tok.getNextTokenI();
	aShip.stealthECCM = tok.getNextTokenI();
	aShip.bpv = tok.getNextTokenI();
	aShip.carrierSizeClass = tok.getNextTokenI();
	aShip.yearFirstAvail = tok.getNextTokenI();
	aShip.yearLastAvail = tok.getNextTokenI();
	aShip.sizeClass = tok.getNextTokenI();
	aShip.geoUI = tok.getNextTokenC();
	aShip.geometry = tok.getNextTokenC();
	aShip.fullName = tok.getNextTokenC();

// Fill in all the things that would otherwise not be filled in
	aShip.hullType = (!aShip.geoUI.compare("XSH")) ? kShuttle : kFighter;
	aShip.shipClass = kClassShuttle;
	aShip.specialRole = kNoSpecialRole;
	aShip.turnMode = kTurnModeAA;
	aShip.hetPlusTwo = 0;
	aShip.hetBreakdown = 0;
	aShip.regularCrew = 0;
	aShip.marinesBase = 1;						// just guessing... :>
	aShip.marinesMax = 1;						// makes sense right?
	aShip.deckCrew = 0;
	aShip.totalCrew = 0;
	aShip.minimumCrew = 0;

	for (int i = (int)kForeShieldIndex; i < (int)kShieldIndexCount; i++)
		aShip.shields[(eShieldIndex)i] = 0;		// just guessing, only PF's have shields
	aShip.totalShielding = 0;
	aShip.cloakCost = Ship::kNoCloak;
	aShip.probes = 0;
	aShip.tBombsBase = 0;
	aShip.tBombsMax = 0;
	aShip.nukeBombsBase = 0;
	aShip.nukeBombsMax = 0;
	aShip.maxDroneControl = 6;					// just guessing
	aShip.add_12 = 0;

	aShip.shuttlesSize = 0;
	aShip.launchRate = 0;
	aShip.adminBase = 0;
	aShip.adminMax = 0;

	aShip.armor = 0;							// probably not
	aShip.forwardHull = 0;
	aShip.centerHull = 0;
	aShip.aftHull = 0;
	aShip.cargo = 0;
	aShip.barracks = 0;
	aShip.repair = 0;
	aShip.leftWarp = 0;
	aShip.centerWarp = 0;
	aShip.rightWarp = 0;
	aShip.impulse = 0;							// probably not
	aShip.apr = 0;
	aShip.battery = 0;
	aShip.bridge = 0;
	aShip.security = 0;
	aShip.lab = 0;
	aShip.transporters = 0;
	aShip.tractors = 0;
	aShip.mechTractors = 0;
	aShip.specialSensors = 0;
	aShip.sensors = 0;							// probably not
	aShip.scanners = 0;
	aShip.explosionStrength = 0;				// probably not
	aShip.acceleration = 0;						// definitely not
	aShip.damageControl = 0;
	aShip.extraDamage = 0;
	aShip.shipCost = 0;
	aShip.balance = 0;


	return true;
}

